@@ -12,7 +12,12 @@ import requests
 
 try:
     import pandas
+    import numpy
     PANDAS_SUPPORT = True
+
+    import re
+    KEY_RE = re.compile(r'\{([\w\d\.]+)}')
+
 except ImportError:
     PANDAS_SUPPORT = False
 
@@ -430,6 +435,8 @@ class Connection(object):
 
 class PandasFormatter(object):
 
+    NULL_VALUE = 'null'
+
     def __init__(self, api_client):
         """
 
@@ -437,7 +444,7 @@ class PandasFormatter(object):
         """
         self.api_client = api_client
 
-    def search_series(self, query='', date_start=None, date_finish=None, size=10, fields=None, query_size=10):
+    def search(self, query='', date_start=None, date_finish=None, size=10, fields=None, query_size=10, key_field=None):
         """
         Performs a block search and returns the results in pandas.DataFrame.
 
@@ -450,10 +457,12 @@ class PandasFormatter(object):
         :return: a pandas.DataFrame
         :rtype: pandas.DataFrame
         """
+        if key_field:
+            fields = self._get_fields_in_key(key_field)
         response = self.api_client.search_series(query, date_start, date_finish, size, query_size, fields)
-        return self._process_search_request(response)
+        return self._process_search_request(response, key_field=key_field)
 
-    def get_points(self, series_id, date_start=None, date_finish=None,size=10):
+    def get_points(self, series_id, date_start=None, date_finish=None, size=10):
         """
         Gets the points of a series
 
@@ -480,7 +489,7 @@ class PandasFormatter(object):
         return pandas.Series(response['fields'])
 
 
-    def _process_search_request(self, response):
+    def _process_search_request(self, response, key_field=None):
         """
         Processes the search results and returns a pandas.DataFrame
 
@@ -496,7 +505,10 @@ class PandasFormatter(object):
         if not series_ids:
             return pandas.DataFrame()
 
-        has_fields = truth(response['fields'].get('fields'))
+        if key_field:
+            fields_for_key = response.pop('fields', {})
+
+        has_fields = truth(response.get('fields', {}).get('fields'))
         has_points = truth(response['points'])
 
         if has_points and has_fields:
@@ -514,8 +526,21 @@ class PandasFormatter(object):
                 {field: list(flatten([value] * num_of_dates for value in response['fields']['values'][i])) for i, field in enumerate(response['fields']['fields'])}
             )
         elif has_points:
-            data = {'series_id': series_ids}
-            data.update({unixtime_to_datetime(date): response['points']['values'][i] for i, date in enumerate(response['points']['dates'])})
+            data = {}
+            if key_field:
+                fields_in_key = self._get_fields_in_key(key_field)
+            for j, series_id in enumerate(series_ids):
+                series = {}
+                for i, dt in enumerate(response['points']['dates']):
+                    if key_field:
+                        key = self._get_column_name(key_field, fields_in_key, fields_for_key, j)
+                    else:
+                        key = series_id
+                    value = response['points']['values'][i][j]
+                    if value is None:
+                        value = numpy.nan
+                    series[unixtime_to_datetime(dt)] = value
+                data[key] = series
 
         elif has_fields:
             data = {'series_id': series_ids}
@@ -527,6 +552,39 @@ class PandasFormatter(object):
 
         return pandas.DataFrame(data)
 
+    def _get_column_name(self, key_field, fields_in_key, fields, series_num):
+        if not fields_in_key:  # no fields were requested, probably user error like {test
+            return self.NULL_VALUE
+
+        # one field case, no format, just return the value fo the field
+        if not '{' in key_field:
+            value = fields['values'][0][series_num]
+            value = value if value is not None else self.NULL_VALUE
+            return value
+
+        field_dict = {}
+        for i, field in enumerate(fields['fields']):
+            value = fields['values'][i][series_num]
+            value = value if value is not None else self.NULL_VALUE
+            if '.' in field:
+                field_sanitized = field.replace('.', '')
+                key_field = key_field.replace(field, field_sanitized)
+                field = field_sanitized
+            field_dict[field] = value
+
+        # fill fields that didn't might not be in the list of fields
+        for field in fields_in_key:
+            if field not in field_dict:
+                field_dict[field] = self.NULL_VALUE
+
+        return key_field.format(**field_dict)
+
+    def _get_fields_in_key(self, key_field):
+        if '{' in key_field:
+            return KEY_RE.findall(key_field)
+        else:
+            return [key_field]
+
 
 class ShoojuApi(object):
     """
